from abc import ABC, abstractmethod
from typing import Any, List, Optional

class MiddlewareInterface(ABC):
    """
    中间件接口
    """
    @abstractmethod
    def process(self, data: Any) -> Any:
        """处理数据并返回处理后的结果"""
        pass

    @abstractmethod
    def get_next(self) -> Optional[List['MiddlewareInterface']]:
        """获取下一个中间件"""
        pass

class EngineInterface(ABC):
    """
    引擎接口
    """
    @abstractmethod
    def execute(self, data: Any) -> Any:
        """执行引擎逻辑并返回结果"""
        pass
    
    @abstractmethod
    def prepare_input(self, data: Any) -> Any:
        """将输入引擎数据剔除元数据"""
        pass

    @abstractmethod
    def process_output(self, data: Any) -> Any:
        """将输出数据转换为引擎的格式"""
        pass

    @abstractmethod
    def to_execute_middleware(self) -> MiddlewareInterface:
        """将引擎具体执行过程转换为遵循MiddlewareInterface的中间件."""
        pass

    @abstractmethod
    def to_prepare_input_middleware(self) -> MiddlewareInterface:
        """将原生引擎数据转换为传输的数据."""
        pass

    @abstractmethod
    def to_process_output_middleware(self) -> MiddlewareInterface:
        """将输出数据包装成原生引擎数据."""
        pass

    @abstractmethod
    def to_middleware(self) -> MiddlewareInterface:
        """将引擎执行流转换为遵循MiddlewareInterface的中间件."""
        pass