from logging import getLogger
from typing import Any, Dict
logger = getLogger(__name__)
import requests

class RemoteEngine():
    """基础远程服务类, 提供远程调用的基础框架。"""
    def __init__(self, api_key: str, base_url: str):
        self._api_key = api_key
        self._base_url = base_url
        self.url = self._base_url
        self.header = {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + self._api_key
        }

    def post_prompt(self, messages : Dict, is_stream : bool = False) -> requests.Response:
        try:
            response = requests.post(self.url, headers=self.header, json=messages, stream=is_stream)
            if (response.status_code != 200):
                logger.error(f"post_prompt error: {response.text}")
                raise Exception(f"post_prompt error: {response.text}")
        except Exception as e:
            logger.exception("post_prompt")
            raise e
        finally:
            return response