# https://dashscope.console.aliyun.com/model
import json
from typing import Any, Generator, Iterator
from .remote_engine import RemoteEngine
from src import ChatAIEngine as ChatAIEngine
from src import StreamEngine as StreamEngine

from logging import getLogger
logger = getLogger(__name__)

class ChatAiTongYiQianWen(ChatAIEngine, RemoteEngine, StreamEngine):
    API_URL = "https://dashscope.aliyuncs.com/api/v1/services/aigc/text-generation/generation"
    def __init__(self, api_key : str,
                 _model : str = "qwen-1.8b-chat",
                 *args, **kwargs) -> None:
        self._modual_name = kwargs.get("model", _model)
        StreamEngine.__init__(self, use_stream_api=kwargs.get("use_stream_api", False), return_as_stream=kwargs.get("return_as_stream", False))
        ChatAIEngine.__init__(self)
        RemoteEngine.__init__(self, api_key=api_key,
                         base_url=self.API_URL)
    def _transform_raw(self, response):
        return response["output"]["choices"][0]["message"]["content"]

    def _transform_iterator(self, response):
        for chunk in response:
            input_string = chunk.decode('utf-8')
            json_string = input_string[input_string.find("data:") + 5:].strip()
            yield self._transform_raw(json.loads(json_string))

    def execute_stream(self, data) -> Generator[Any, None, None] | Iterator:
        self.header['X-DashScope-SSE'] = 'enable'
        response = self.post_prompt({
                'model': self._modual_name,
                "input": {
                    "messages": data
                },
                "parameters": {
                    "result_format": "message",
                    "incremental_output": True
                }
            },
            is_stream=True)

        ret = response.iter_content(chunk_size=None)
        return self._transform_iterator(ret)
    
    def execute_nonstream(self, data) -> Any:
        response = self.post_prompt({
                'model': self._modual_name,
                "input": {
                    "messages": data
                },
                "parameters": {
                    "result_format": "message",
                    "incremental_output": False
                }
            },
            is_stream=False)
        ret = response.json()
        return self._transform_raw(ret)

    def prepare_input(self, data: Any) -> Any:
        data['__data__'] = {'role': 'user', 'content': f'{data['__data__']}'}
        return super().prepare_input(data)